import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;

public class Worm {
	// speed of the worm (1 dot-move after SPEED millis)
	private static final int SPEED = 100;
	private long lastMoveTime = 0;	// time of last movement
	
	// size and number of dots in a worm
	public static final int DOTSIZE = 15;
	public static final int RADIUS = DOTSIZE / 2;
	private static final int MAXDOTS = 40;
	private static final int INITIALDOTS = 5;

	// compass direction/bearing constants
	public static final int NORTH = 0; // north, etc going clockwise
	public static final int EAST = 1;
	public static final int SOUTH = 2;
	public static final int WEST = 3;
	private int currentDirection; // stores the movement-direction

	// Tells how "fast" the worm is
	private static final int PIXEL_MOVE = DOTSIZE;
	
	private ArrayList<Point> dots; // stores dots of the worm (head is last)
	private int pWidth, pHeight; // panel dimensions
	
	public Worm(int pW, int pH) {
		pWidth = pW;
		pHeight = pH;
		dots = new ArrayList<Point>(MAXDOTS);
		initializeWorm();
	} // end of Worm()


	private void initializeWorm() {
		currentDirection = EAST; // set initial worm direction
		int x = pWidth / 2;		 // generate dots (starting in center)
		int y = pHeight / 2;
		for (int i=0 ; i < INITIALDOTS-1; i++) {
			dots.add(0, new Point(x,y));
			x = x - DOTSIZE;
		}
	}

	
	public void move(int newDirection) {
		
		if ((System.currentTimeMillis() - lastMoveTime) < SPEED) {
			return;
		}
		lastMoveTime = System.currentTimeMillis();

		// skip movements in opposite direction of current movement-direction
		boolean setDirection = true;
		if (newDirection == WEST && currentDirection == EAST) setDirection = false;
		if (newDirection == EAST && currentDirection == WEST) setDirection = false;
		if (newDirection == NORTH && currentDirection == SOUTH) setDirection = false;
		if (newDirection == SOUTH && currentDirection == NORTH) setDirection = false;
		if (setDirection) currentDirection = newDirection;
		
		// remove tail and add head 
		dots.remove(0);
		addHead();
		
	} // end of move()


	public void addHead() {
		// calculate movement 
		int xMove = 0;
		int yMove = 0;
		if (currentDirection == NORTH) {
			xMove = 0;
			yMove = PIXEL_MOVE * -1;
		} else if (currentDirection == SOUTH) {
			xMove = 0;
			yMove = PIXEL_MOVE * 1;
		} else if (currentDirection == EAST) {
			xMove = PIXEL_MOVE * 1;
			yMove = 0;
		} else if (currentDirection == WEST) {
			xMove = PIXEL_MOVE * -1;
			yMove = 0;
		}
		
		// calculate new position based on movement and heads position
		Point head = dots.get(dots.size()-1);
		int xNew = head.x + xMove;
		int yNew = head.y + yMove;

		// worm moves over borders?
		if (xNew > pWidth) xNew = RADIUS;
		if (xNew < 0) xNew = pWidth;
		if (yNew > pHeight) yNew = RADIUS;
		if (yNew < 0) yNew = pHeight;
		
		// create new head (end of list)
		Point newHead = new Point(xNew, yNew);
		dots.add(newHead);
	}

	
	public boolean nearHead(int x, int y)
	// collission-detection: is (x,y) near the worm's head?
	{
		if (dots.size() > 0) {
			int headPos = dots.size() - 1;
			if ((Math.abs(dots.get(headPos).x + RADIUS - x) <= DOTSIZE)
					&& (Math.abs(dots.get(headPos).y + RADIUS - y) <= DOTSIZE))
				return true;
		}
		return false;
	} // end of nearHead()

	
	public boolean ateMe()
	// collission detection: is head near any part of own body?
	{
		Point head = dots.get(dots.size()-1);
		int i = 0;
		while (i != (dots.size() - 1)) {
			if ((Math.abs(dots.get(i).x + RADIUS - head.x) <= RADIUS)
					&& (Math.abs(dots.get(i).y + RADIUS - head.y) <= RADIUS))
				return true;
			i = (i + 1) % MAXDOTS;
		}
		return false;
	} // end of touchedAt()

	
	public void draw(Graphics g)
	// draw a black worm with a red head
	{
		if (dots.size() > 0) {
			g.setColor(Color.black);
			int i = 0;
			while (i != (dots.size()-1)) {
				g.fillOval(dots.get(i).x, dots.get(i).y, DOTSIZE, DOTSIZE);
				i = (i + 1) % MAXDOTS;
			}
			g.setColor(Color.blue);
			Point head = dots.get(dots.size()-1);
			g.fillOval(head.x, head.y, DOTSIZE, DOTSIZE);
		}
	} // end of draw()

} // end of Worm class

